#!/usr/bin/perl -w

use Tk;
use Tk::LabFrame;
use FileHandle;
use IPC::Open2;
use XML::Simple;
use strict;
use Encode qw/encode decode/;
our ($m,$path,$n_pagenav,$n_sort,$n_recalc,$m_maxwidth,$t_title,$i_noimageindex,$x_noextracttitle,$version,$thumb_x,$thumb_y,$q_be_quiet,$hlp,$codepage);
if ($^O =~ 'MSWin') {
   $codepage='cp1252';
} else {
   $codepage='iso-8859-1';
}
$version='1.04';
$m = MainWindow->new(-title => "CD2Browse GUI" );
if ( $^O =~ /win/i && $^X =~ /wperl/ ) {
   displayError("This Script must not run under wperl\n Make sure to use perl instead");
   $m->destroy;
   exit (1);
}
my $menu=$m->Frame( -relief=>"raised",
                    -borderwidth=>2);
my $canvas=$m->Canvas();
my $row=0;
my $frame=$canvas->LabFrame(-label=>"Path to index", -labelside=>"acrosstop");
my $dirbutton=$frame->Button(-command => sub {choosedir()},
                              -text => "Choose Directory")->pack(-anchor=>'e', -side=>'right');
my $dirfield=$frame->Entry(-width=>80,-textvariable=>\$path)->pack(-anchor=>'w',-side=>'left');
$frame->pack;
my $frame2=$canvas->LabFrame(-label=>"Options (no option is mandatory - all fields may stay blank)", -labelside=>"acrosstop");
my $titlelabel=$frame2->Label(-text=>'Title text',-anchor => 'e', -justify => 'right');
my $titleentry=$frame2->Entry(-width=>71, -textvariable=>\$t_title);
$titlelabel->grid(-row=>$row, -column=>0,-sticky=>'e',-pady=>3,-padx=>3);
$titleentry->grid(-row=>$row++, -column=>1,-sticky=>'w',-columnspan=>8);
my $widthlabel=$frame2->Label(-text=>'Max image width (in pixel)',-anchor => 'e', -justify => 'right');
my $widthentry=$frame2->Entry(-width=>4, -textvariable=>\$m_maxwidth);
$widthentry->bind('<FocusOut>',[sub {widthcheck()}]);
$widthlabel->grid(-row=>$row, -column=>0,-sticky=>'e',-pady=>3,-padx=>3);
$widthentry->grid(-row=>$row++, -column=>1,-sticky=>'w',-pady=>3);
my $thumblabel=$frame2->Label(-text=>'Thumb size (in pixel)',-anchor => 'e', -justify => 'right');
my $thumb_x_entry=$frame2->Entry(-width=>3, -textvariable=>\$thumb_x);
my $thumb_mul=$frame2->Label(-text=>' X ');
my $thumb_y_entry=$frame2->Entry(-width=>3, -textvariable=>\$thumb_y);
$thumb_x_entry->bind('<FocusOut>',[sub {thumbcheck()}]);
$thumb_y_entry->bind('<FocusOut>',[sub {thumbcheck()}]);
$thumblabel->grid(-row=>$row, -column=>0,-sticky=>'e',-pady=>3,-padx=>3);
$thumb_y_entry->grid(-row=>$row, -column=>1,-sticky=>'w',-pady=>3);
$thumb_mul->grid(-row=>$row, -column=>1,-sticky=>'s',-pady=>3);
$thumb_x_entry->grid(-row=>$row++, -column=>1,-sticky=>'e',-pady=>3);
my $indexlabel=$frame2->Label(-text=>'Do not index images',-anchor => 'e', -justify => 'right');
my $indexcheck=$frame2->Checkbutton(-variable=>\$i_noimageindex);
$indexlabel->grid(-row=>$row, -column=>0,-sticky=>'e',-padx=>3);
$indexcheck->grid(-row=>$row++, -column=>1,-sticky=>'w');
my $pdflabel=$frame2->Label(-text=>'Do not read titles from PDF',-anchor => 'e', -justify => 'right');
my $pdfcheck=$frame2->Checkbutton(-variable=>\$x_noextracttitle);
$pdflabel->grid(-row=>$row, -column=>0,-sticky=>'e',-padx=>3);
$pdfcheck->grid(-row=>$row++, -column=>1,-sticky=>'w');
my $recalclabel=$frame2->Label(-text=>'Do not recalculate images',-anchor => 'e', -justify => 'right');
my $recalccheck=$frame2->Checkbutton(-variable=>\$n_recalc);
$recalclabel->grid(-row=>$row, -column=>0,-sticky=>'e',-padx=>3);
$recalccheck->grid(-row=>$row++, -column=>1,-sticky=>'w');
my $sortlabel=$frame2->Label(-text=>'Do not create sort pages',-anchor => 'e', -justify => 'right');
my $sortcheck=$frame2->Checkbutton(-variable=>\$n_sort);
$sortlabel->grid(-row=>$row, -column=>0,-sticky=>'e',-padx=>3);
$sortcheck->grid(-row=>$row++, -column=>1,-sticky=>'w');
my $pagenavlabel=$frame2->Label(-text=>'Do not create folder navigation in page section',-anchor => 'e', -justify => 'right');
my $pagenavcheck=$frame2->Checkbutton(-variable=>\$n_pagenav);
$pagenavlabel->grid(-row=>$row, -column=>0,-sticky=>'e',-padx=>3);
$pagenavcheck->grid(-row=>$row++, -column=>1,-sticky=>'w');
my $quietlabel=$frame2->Label(-text=>'Be quiet - no progress info',-anchor => 'e', -justify => 'right');
my $quietcheck=$frame2->Checkbutton(-variable=>\$q_be_quiet);
$quietlabel->grid(-row=>$row, -column=>0,-sticky=>'e',-padx=>3);
$quietcheck->grid(-row=>$row, -column=>1,-sticky=>'w');
$frame2->pack;
my $start=$canvas->Button(-command => sub {process_path()},
                          -text => "Start Indexing")->pack(-anchor=>'s', -side=>'bottom');
$canvas->pack(-side=>'bottom');
my $filebar=$menu->Menubutton(-text=>"File",
                     -underline => 0)->pack;
my $helpbar=$menu->Menubutton(-text=>"Help",
                     -underline => 0)->pack;
my $filemenu = $filebar->Menu();
$filebar->configure(-menu=>$filemenu);
my $helpmenu = $helpbar->Menu();
$helpbar->configure(-menu=>$helpmenu);
$helpmenu->command(-command => sub {openAbout()},
                     -label => "About",
                     -underline => 0);
$helpmenu->command(-command => sub {show_help()},
                     -label => "Help",
                     -underline => 0);
$filemenu->command(-command => sub {open_set()},
                     -label => "Open setting",
                     -underline => 0);
$filemenu->command(-command => sub {save_set()},
                     -label => "Save setting",
                     -underline => 0);
$filemenu->separator;
$filemenu->command(-label => "Exit",
                     -command => sub {exit_main()},
                     -underline => 1);
$filebar->pack(-side=>"left");
$helpbar->pack(-side=>"left");
$menu->pack(-side=>"top",
                 -fill=>"x");
MainLoop;
exit(0);
#--------------------------------------------------------
# Check if the width of the images in within a certain size range
#--------------------------------------------------------
sub thumbcheck {
   if ($thumb_x && ($thumb_x > 200 || $thumb_x < 10 )) {
      displayError("The thumb nail width must be in between 10 and 200")
   }
   if ($thumb_y && ($thumb_y > 200 || $thumb_y < 10 )) {
      displayError("The thumb nail height must be in between 10 and 200")
   }
}
#--------------------------------------------------------
# Check if the width of the images in within a certain size range
#--------------------------------------------------------
sub widthcheck {
   if ($m_maxwidth && ($m_maxwidth > 2000 || $m_maxwidth < 20 )) {
      displayError("The image width must be in between 20 and 2000")
   }
}
#--------------------------------------------------------
# Starts CD2Browse in a command window
#--------------------------------------------------------
sub process_path {
   if ( ! $path ){displayError("No path !\nchoose a path first"); return}
   my $options= "-p \"$path\" ";
   $options.='-n ' if $n_recalc;
   $options.='-r ' if $n_sort;
   $options.='-v ' if $n_pagenav;
   $options.="-m $m_maxwidth " if $m_maxwidth;
   $options.="-s $thumb_x".'x'.$thumb_y.' ' if ($thumb_x and $thumb_y);
   $options.="-t \"$t_title\" " if $t_title;
   $options.='-i ' if $i_noimageindex;
   $options.='-x ' if $x_noextracttitle;
   $options.='-q' if $q_be_quiet;
   my $t = MainWindow->new(-title => "index.pl output" );
   my $tx = $t->Text(-width => 80, -height => 50, -wrap => 'char');
   $tx->pack(-expand => 1);
   my ($chld_out,$chld_in)=('out','in');
   print "Start Processing with command: perl index.pl $options\n\n";
   my $pid = open2($chld_out, $chld_in, 'perl index.pl '.$options);
   $t->repeat(50=>[\&fill_text_widget,$tx,$chld_out,$pid]);
}
#--------------------------------------------------------
# Fills the process info text window with output of index.pl
#--------------------------------------------------------
sub fill_text_widget {
   my $widget = shift;
   my $chld_out = shift;
   my $pid = shift;
   if (!Exists ($widget)) {waitpid $pid, 0; return}
   $_ = <$chld_out>;
   if ( $_ ){
      $_=~tr/\r//d;
      $widget->insert('end', $_);
      $widget->yview('end');
   }
}
#--------------------------------------------------------
# opens a chooser for the path and sets the global var
#--------------------------------------------------------
sub choosedir {
   $path=$m->chooseDirectory;
}
#--------------------------------------------------------
# Opens a setting file and loads the fields
#--------------------------------------------------------
sub open_set {
   my $file=$m->getOpenFile( -filetypes => [['Setting files',['.set','.xml']],['All Files',['*',]]],
                             -defaultextension => '.set');
   $file=encode($codepage,$file);
   my $settings=XMLin($file, forcearray => 1,SuppressEmpty => 1);
   $path=$settings->{path}->[0];
   $n_recalc=$settings->{n_recalc}->[0];
   $n_sort=$settings->{n_sort}->[0];
   $n_pagenav=$settings->{n_pagenav}->[0];
   $m_maxwidth=$settings->{m_maxwidth}->[0];
   $t_title=$settings->{t_title}->[0];
   $i_noimageindex=$settings->{i_noimageindex}->[0];
   $x_noextracttitle=$settings->{x_noextracttitle}->[0];
   $version=$settings->{version}->[0];
   $thumb_x=$settings->{thumb_x}->[0];
   $thumb_y=$settings->{thumb_y}->[0];
   $q_be_quiet=$settings->{q_be_quiet}->[0];
}
#--------------------------------------------------------
# Saves the current setting into a setting file
#--------------------------------------------------------
sub save_set {
   my $file=$m->getSaveFile( -filetypes => [['Setting files',['.set','.xml']],['All Files',['*',]]],
                             -defaultextension => '.set');
   my $settings={path            =>[$path],
                 n_recalc        =>[$n_recalc],
                 n_sort          =>[$n_sort],
                 n_pagenav       =>[$n_pagenav],
                 m_maxwidth      =>[$m_maxwidth],
                 t_title         =>[$t_title],
                 i_noimageindex  =>[$i_noimageindex],
                 x_noextracttitle=>[$x_noextracttitle],
                 version         =>[$version],
                 thumb_x         =>[$thumb_x],
                 thumb_y         =>[$thumb_y],
                 q_be_quiet      =>[$q_be_quiet]};
   $file=encode($codepage,$file);
   my $xml=XMLout($settings,OutputFile=>$file,RootName => 'config',SuppressEmpty => 1);
}
#--------------------------------------------------------
# The About popup
#--------------------------------------------------------
sub openAbout {
    displayInfo("CD2Browse GUI\n    written by\n   Rolf Jethon\n");
  }
#--------------------------------------------------------
# Show the help window containing a quick guide
#--------------------------------------------------------
sub show_help {
  if (Exists ($hlp)) {$hlp->destroy}
  my $HELP = "CD2Browse.hlp";
  my $fh = new FileHandle($HELP,"<");
  if ( defined $fh ){
     $hlp = MainWindow->new(-title => "CD2Browse Help" );
     my $OLDHEAD = $hlp->Text(-height     => 60,
                              -width      => 80,
                              -wrap       => 'word');
     $OLDHEAD->pack(-anchor => "c",
                    -expand => 1,
                    -fill => "both",
                    -padx => 5,
                    -pady => 5,
                    -side => "top");
     $OLDHEAD->delete("0.0","end");
     while (<$fh>) {$OLDHEAD->insert("end",$_);}
     } else {displayError("Could not open file $HELP");
  }
}
#--------------------------------------------------------
# Display a info message
#--------------------------------------------------------
sub displayInfo {
my $inf=shift;
require Tk::Dialog;
my @buttons = ('OK');
my $buttonRef = \@buttons;
my $dialog=$m->Dialog(-bitmap => 'info',
                               -buttons => $buttonRef,
                               -text => $inf,
                               -title => "Note:");
$dialog->Show();
}
#--------------------------------------------------------
# Display a error message
#--------------------------------------------------------
sub displayError {
my $err=shift;
require Tk::Dialog;
my @buttons = ('OK');
my $buttonRef = \@buttons;
my $dialog=$m->Dialog(-bitmap => 'error',
                               -buttons => $buttonRef,
                               -text => $err,
                               -title => "Error:");
$dialog->Show();
}
#--------------------------------------------------------
# convert a string to a decimal list
#--------------------------------------------------------
sub string_2_decimal {
   my $string=shift;
   my $asc;
   for ( my $a=0;length($string)> $a ;$a++) {
      $asc.=ord(substr($string,$a)).' ';
   }
   chomp $asc;
   return ($asc);
}
#--------------------------------------------------------
# determine the system encoding
#--------------------------------------------------------
sub get_encoding {
   if ($^O eq 'MSWin32'){
      require Win32::Registry;
      my $r;
      $::HKEY_LOCAL_MACHINE->Open('SYSTEM\CurrentControlSet\Control\NLS\CodePage', $r)
      or die "Can't open
      HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\NLS\\CodePage:
      $^E\n";
      my $SystemEncoding;
      $r->GetValue('ACP', $SystemEncoding);
      $SystemEncoding = '1252' unless $SystemEncoding;
      $SystemEncoding = 'cp' . $SystemEncoding;
      $::HKEY_LOCAL_MACHINE->Close();
      return $SystemEncoding;
   }
}
#--------------------------------------------------------
# exit the main program
#--------------------------------------------------------
sub exit_main {
   exit(0);
}