#!/usr/bin/perl -w
# Program written by Rolf Jethon 15 Oct 2007
# Modified (1.01) 28 Dec 2007 (RJ)
# Modified (1.02) 01.Oct 2008 (RJ)
# Modified (1.03) 16.Nov 2009 (RJ)
# Modified (1.04) 01.Aug 2010 (RJ)
# Modified (1.05) 13.Jul 2013 (RJ)
#
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.

#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
#    cd2browse is written in the hope, that it is helpful
#-------------------------------------------------------------------------------
# cd2browse creates a browsable HTML Menu out of any file tree, adressed on the command line
# version 1.05

use strict;
use Module::Load::Conditional qw[check_install];
use Image::Magick;
use File::Copy;
use File::Mirror;
use CGI::FastTemplate;
use Getopt::Std;
my $h_strip_exist= check_install( module => 'HTML::Strip');
my $use_extract=0;
if ( ! $h_strip_exist->{uptodate} ) {
  require HTML::ExtractContent;
  $use_extract=1 ;
} else { require HTML::Strip;
}
my $mp3info_exist= check_install( module => 'MP3::Info');
my $has_mp3info=0;
if ( $mp3info_exist->{uptodate} ) {
  require MP3::Info;
  $has_mp3info=1 ;
}
use Encode qw/encode decode/;
$|=1; # force print not to use output buffers
getopts ('t:s:m:p:nixrqv');
our ($opt_p,$opt_n,$opt_t,$opt_s,$opt_m,$opt_i,$opt_x,$opt_r,$opt_q,$opt_v,$datahome,$codepage);
if ($^O =~ 'MSWin') {
   $codepage='cp1252';
} else {
   $codepage='iso-8859-1';
}
$opt_p=encode($codepage,decode("utf-8",$opt_p));
if ( ! $opt_p ) {
print "usage: perl index.pl -p <path to index> [-n -m -t -s -i -x -r -q -v]

Options:
  -i  do not index images (all over the file tree)
  -m  maxwidh of the preview image (default=800)
  -n  (re) calculate no images to save cpu time
  -s  size of the thumbnails (default=90x90)
  -t  title
  -q  be quiet - do not output progress information
  -r  do not use offline sort - needs a different template page.tpl
  -v  do not create folder navigation in the page section
  -x  do not extract title from pdf file (all over the file tree)


";
exit;
}
#---------------- global settings - you may change here ! ----------------------
my $thumbsize='90x90';                            # max size of the thumbnail images
my $title='CD 2 Browse HTML Men';                # the main title of the new documentation
if ($ENV{C2BPATH}) {
  $datahome=$ENV{C2BPATH};
} else {
  if ($^O =~ /win/i) {                              # assume the windows resources are on c:, so include it in the path
     $datahome="C:/var/opt/cd2browse/repository";   # This is the main dir where the resources are stored
  } else {                                          # unix needs no drives ;-)
     $datahome="/var/opt/cd2browse/repository";     # This is the main dir where the resources are stored
  }
}
if (! -e "$datahome/result.html" ) { 
  print STDERR "base repository could not be found - please check the C2BPATH Environment Variable\n"; exit 0;
}
my $templatehome="$datahome/templates";           # here are the source templates for the html gen.
my $resourcesource="$datahome/resources";         # the source resources for all projects (images and some static html pages)
my $menuresource="$datahome/menu";                # here the tree menu resources reside
my $repopath=".repository";                         # here go all repository files
my $outpath="$repopath/html";                       # here go the browsable html output files
my $resourcedir="$repopath/resources";              # this is the local resource instance for the project site
my $templatedir="$repopath/templates";              # here reside the local project temlates
my $menudir="$repopath/menu";                       # here resides the local project JS menu
my $menutemplatedir="$menudir/templates";           # here are the templates for the local tree menu
my $thumbdir="$repopath/thumb";                     # dir, where the local thumbnails are stored
my $indexfile="$repopath/index.js";                 # the index file for the js Index
my $maxwidth=800;                                   # maximum default image width for the previews
my $has_pdftotext=1;   # if 1 -> enables pdf indexing (external binary pdftotext necessary)
my $has_catdoc=1;      # if 1 -> enables word, excel and power point indexing (ext. binary catdoc, xls2csv and catppt necessary)
#-------------------------------------------------------------------------------
$maxwidth=$opt_m if $opt_m;
$title=$opt_t if $opt_t;
$thumbsize=$opt_s if $opt_s;
print "Please be patient -
the calculation takes time ..
";
my $path=$opt_p;
$path=~s#\\#/#g;
my $success=chdir $path;
if (! $success) { print "Could not change directory to: $path $!"; exit;}
if ( ! -d "$path/$repopath" ) {mkdir "$path/$repopath"}
if ( ! -d "$path/$outpath" ) {mkdir "$path/$outpath"}
if ( ! -d "$path/$thumbdir" ) {mkdir "$path/$thumbdir"}
#------------ copy the resources to the path, if not there already -------------
mirror $resourcesource, $resourcedir if ! -d "$path/$resourcedir";
mirror $menuresource, $menudir if ! -d "$path/$menudir";
mirror $templatehome, $templatedir if ! -d "$path/$templatedir";
copy "$datahome/head.html","$repopath/head.html" if ! -e "$path/$repopath/head.html";
copy "$datahome/welcome.html","$repopath/welcome.html" if ! -e "$path/$repopath/welcome.html";
copy "$datahome/start.exe",'start.exe' if ! -e "$path/start.exe";
copy "$datahome/autorun.inf",'autorun.inf' if ! -e "$path/autorun.inf";
copy "$datahome/cd.ico",'cd.ico' if ! -e "$path/cd.ico";
copy "$datahome/index.html",'index.html' if ! -e "$path/index.html";
copy "$datahome/result.html","$repopath/result.html" if ! -e "$path/$repopath/result.html";
#-------------------------------------------------------------------------------

%main::icons=(
'pdf','pdf.gif',
'doc','doc.gif',
'zip','compressed.gif',
'dwg','unknown.gif',
'log','text.gif',
'xml','text.gif',
'bat','script.gif',
'exe','sphere2.gif',
'dll','screw2.gif',
'txt','text.gif',
'html','world2.gif',
'htm','world2.gif',
'tar','tar.gif',
'mid','sound2.gif',
'dvi','dvi.gif',
'ps','ps.gif',
'mp3','sound1.gif',
'avi','movie.gif',
'mov','movie.gif',
'dir','dir.gif',
'bk','back.gif'
);

my $looplimit=64;
my ($j,$i,$k,$thumbfile)=(0,0,0);
open ( INDEX,">$indexfile")||die "Could'nt open file for write: $path/$indexfile - $!";
print INDEX "var item = new Array();\nc=0; ";
@main::tree=();          # empty the treemenu
&makemenu();
close (INDEX);
END: print "\nIndexing done - Program finished !\n";
#----------------------------------------------------------
# create the menu
#----------------------------------------------------------
sub makemenu {
      $main::tpl=new CGI::FastTemplate($menutemplatedir);
      $main::tpl->define(main  =>'tree.tpl',
                         menu1_1 =>'treeitem1-1.tpl',
                         menu1_2 =>'treeitem1-2.tpl',
                         menu2_1 =>'treeitem2-1.tpl',
                         menu2_1a=>'treeitem2-1a.tpl',
                         menu2_2 =>'treeitem2-2.tpl',
                         menu2_2a=>'treeitem2-2a.tpl'
                         );
      $main::levels=0;
      my @empty;
      copy ("$resourcedir/image_end.html","$thumbdir/000000.html");
      plane('.',\@empty,0,'../welcome');
      my $last=get_zero_lead ($i+1);
      push @main::tree, [0,$main::levels,0,0];
      copy ("$resourcedir/image_end.html","$thumbdir/$last.html");
      write_tree();
      $main::tpl->assign(TITLE =>$title);
      $main::tpl->parse(MAIN   =>'main');
      my $content=$main::tpl->fetch('MAIN');
      open (OUT,">$repopath/tree.html")|| print "Creating tree menu\nError\nCould'nt open tree.html for writing\n".
      "Please check if the path exists and if the path is writable" && goto END;
      print OUT $$content;
      close (OUT);
}
#----------------------------------------------------------
# fill the tree menu template with the file tree structure recorded during robot run
#----------------------------------------------------------
sub write_tree {
  my $arrow;
  my $closearrow;
  my $previousfragment=shift @main::tree;               # contains the tree info
  my $oldfragment=$previousfragment;
  foreach my $treefragment (@main::tree) {
  $main::tpl->assign(SPACE =>' ' x ($previousfragment->[1] * 2+6));
    if ($previousfragment->[0]==1) {    # its going up the tree, so open a new menu entry
      $main::tpl->assign(ADR   =>"html/$previousfragment->[2].html",
                         LNAME =>$previousfragment->[3]);
      if ($treefragment->[0]==0) {      # this branch has no subdirs - so do not add a arrow
        $arrow=0;
      } else { $arrow=1 }
      if ( $previousfragment->[1] >= 2 ) {              # this construct makes sure, you can use 2 level tree menus
        if ( $arrow ) {
          $main::tpl->parse(MENU   =>'.menu2_1a');      # the template with the arrow
        } else {
          $main::tpl->parse(MENU   =>'.menu2_1');       # the first level is the base structure from treeitem1-1 and 1-2
        }
      } else {                                          # the second level the next higher for the tree menus
        $main::tpl->parse(MENU   =>'.menu1_1');         # splitting of the templates is necessary to allow nesting
      }                                                 # of sub directory levels
    } else {                                            # its going down the tree - so close a menu entry
      if ($oldfragment->[1] > $previousfragment->[1]) { # means we come from a sub directory
        $closearrow=1;
      } else {
        $closearrow=0;
      }
      $main::tpl->assign(SPACE =>' ' x ($previousfragment->[1] * 2+6));   # fill with spaces to get nice looking code indents
      if ( $previousfragment->[1] >= 2 ) {
        if ( $closearrow ) {
          $main::tpl->parse(MENU   =>'.menu2_2a');
        } else {
          $main::tpl->parse(MENU   =>'.menu2_2');
        }
      } else {
        $main::tpl->parse(MENU   =>'.menu1_2');
      }
    }
       $oldfragment=$previousfragment;
       $previousfragment=$treefragment;
  }
}
#----------------------------------------------------------
# create the tree menu items - calls itself - depending on the tree depth
#----------------------------------------------------------
sub plane {
   my $item=shift;
   my $omit_previous=shift;
   my $notitle=shift;
   my $father=shift;
   my @ht_entries;
   my @omit_pattern=@$omit_previous;            # keeps later extensions, that will be omitted
   print "analysing dir: $item\n" if ! $opt_q;
   if ($main::levels++ > $looplimit) {print "Creating tree menu\nError\nThe maximum number of sub-planes is exceeded\n".
     "Most likely a endless loop exist in the file tree\n"; goto END;
   }
   opendir (DIR,"$item")||die "couldn't open file $item - $!";
   if ((stat ("$item/.htaccess"))[7]) {         # .htaccess exists -so we read the content
     open (HT,"$item/.htaccess")||die "couldn't open file $item/.htaccess - $!";
     push @ht_entries,<HT>;
     close (HT);
   }
   foreach my $line (@ht_entries) {
     if ( $line =~/^ *IndexIgnore +(.*)/i ) {
       $_=$1;
       s/\./\\\./g;
       s/\*/\.\*/g;
       push @omit_pattern,split /\s+/, $_;
       # skip indexing per directory if in .htaccess
     }
     if ( $line =~/^ *IndexOptions +SuppressDescription/i ) {
       $notitle=1;
     }
   }
   @omit_pattern=sort(@omit_pattern);             # remove double entries - start
   my $prev;
   my @new;
   foreach my $pattern (@omit_pattern) {
     if ( $prev ne $pattern ) { push @new,($pattern) }
     $prev=$pattern;
   }
   @omit_pattern=@new;                            # remove double entries - end
   my @plane=readdir DIR;
   closedir(DIR);
   my $htmlname=expand();
   $j++;
   my $dir_has_files=0;
   my $mgk=Image::Magick->new;
   my ($tag,$inf);
   my @planecontent;
   my ($linkname,$age,$size,$im_marker,$rawsize,$rawtime);
   plane: foreach my $plane_item (@plane) {
     foreach my $pattern (@omit_pattern) {
       if ( $plane_item=~/^$pattern$/i ) { next plane }                  # skip the files fitting to the pattern
     }
     if ( $plane_item =~ /^\./ ) {next }
     if ( -d "$item/$plane_item" ) {                                     # its a directory - so write the stuff for the tree menu
       my $nextname=expand();
       push @main::tree, [1,$main::levels,$nextname,$plane_item];        # save the branch info before entering a new directory
       plane( "$item/$plane_item",\@omit_pattern,$notitle,$htmlname );   # enter the new found sub directory
       push @main::tree, [0,$main::levels,0,0];                          # save the branch info after the robot comes back
       if ( $opt_v ) { next }                                            # skip directory entry creation
       my ($linkname,$age,$size,$im_marker,$rawsize,$rawtime);
       ($linkname,$tag,$inf)=get_description($item,$plane_item,\@ht_entries,$notitle);
       $thumbfile=get_icon("$nextname.dir",1);                           # fake its a dir
       push @planecontent, [$thumbfile,get_status($item,$plane_item),"$nextname.html",$linkname];
     } else {                                                            # if its a file, it goes here
       if ( $main::levels == 1 ) { next }                                # dont index files in the doc root
       if ($plane_item=~/(\.jpe*g$|\.gif$|\.tif$|\.bmp$|\.png$)/i) {     # its a image - create thumb image an reduced image
         if ( $opt_i ) { next }                                          # skip image indexing
         $i++;
         $im_marker=1;
         ($linkname,$age,$size)=process_image($mgk,$i,$item,$plane_item,\@planecontent,\@ht_entries);
       } else {                                                          # its an ordinary file
         $im_marker=0;
         ($linkname,$tag,$inf)=get_description($item,$plane_item,\@ht_entries,$notitle);
         $thumbfile=get_icon($plane_item);
         ($size,$age)=get_status($item,$plane_item);
         push @planecontent, [$thumbfile,get_status($item,$plane_item),"../.$item/$plane_item",$linkname];
       }
       $dir_has_files++;
       add_to_js_index($linkname,$item,$plane_item,$age,$size,$im_marker,$tag,$inf); # add a entry to the index for the java script search engine
     }
   }
   undef $mgk;
   write_dirpage($htmlname,$dir_has_files,$item,\@planecontent,$father);
   $main::levels--;
}
#----------------------------------------------------------
# expand numbers to 6 digits
#----------------------------------------------------------
sub expand {
  $_='00000'.$j;
  /(.{6}$)/;
  $_=$1;
  return $_;
}
#----------------------------------------------------------
# write the page(s) for the processed directory - if sorting is on write several
#----------------------------------------------------------
sub write_dirpage {
  my $htmlname=shift;
  my $dir_has_files=shift;
  my $item=shift;
  my $planecontentref=shift;
  my $father=shift;
  my $content;
  if ( ! $dir_has_files && $opt_v ) {  # if so, dont create empty pages - instead create a empty message page
    open (EMPTY,"<$resourcedir/empty_dir.html")||die "Could'nt open $outpath/$htmlname.html - $!";
    read EMPTY,my $cnt,65535;
    $content=\$cnt;
    close (EMPTY);
    open (OUT,">$outpath/$htmlname.html")||die "Could'nt open $outpath/$htmlname.html - $!";
    print OUT $$content;
    close (OUT);
  } else {
    my @sortparam=(
    [3,0,'','i','s','ar','n'],    # -> default - used from all other links except the switchbar
    [0,0,'i','ir','s','','n'],    # The sorting feature allows by clicking a switchbar link, to see
    [0,1,'ir','i','s','','n'],    # the content sorted in different ways.
    [3,1,'ar','i','s','','n'],    # Here we define the parameters for the different sortings
    [4,0,'s','i','sr','','n'],    # icon,forward sort,marker which one, 4 fields with extensions
    [4,1,'sr','i','s','','n'],    # to be used in the switchbar
    [6,0,'n','i','s','','nr'],
    [6,1,'nr','i','s','','n']);
    foreach my $paramline (@sortparam){
      if ( $paramline->[2] eq 's'||$paramline->[2] eq 'sr'||$paramline->[2] eq ''||$paramline->[2] eq 'ar' ) {
        @$planecontentref = sort {lc($a->[$paramline->[0]]) <=> lc($b->[$paramline->[0]])} @$planecontentref;
      } else {
        @$planecontentref = sort {lc($b->[$paramline->[0]]) cmp lc($a->[$paramline->[0]])} @$planecontentref;
      }
      if ( $paramline->[1] ) { @$planecontentref=reverse @$planecontentref };

      my $htm=new CGI::FastTemplate($templatedir);
      $htm->define(main =>'page.tpl',
                   list =>'list.tpl');
      $htm->assign(PATH   =>$item,
                   LICON =>$htmlname.$paramline->[3].'.html',
                   LSIZE =>$htmlname.$paramline->[4].'.html',
                   LAGE  =>$htmlname.$paramline->[5].'.html',
                   LTITLE=>$htmlname.$paramline->[6].'.html',
                   BACK  =>"$father.html");
      if ( ! $dir_has_files && ! $planecontentref->[0]->[0]) {    # if the dir is empty put a null string into LIST
      	$htm->assign(LIST    =>"");
      } else {             
        foreach (@$planecontentref){
          $htm->assign(ICON    =>$_->[0],               #$thumbfile
                       SIZE    =>$_->[1],               #$size
                       AGE     =>$_->[2],               #$age
                       LINK    =>$_->[5],               #.$item/$plane_item
                       LINKNAME=>$_->[6] );             #$linkname
          $htm->parse( LIST =>".list");
        }
      }
      $htm->parse( MAIN =>"main");
      $content=$htm->fetch("MAIN");
      undef $htm;
      open (OUT,">$outpath/$htmlname$paramline->[2].html")||die "Could'nt open $outpath/$htmlname.html - $!";
      print OUT $$content;
      close (OUT);
      if ( $opt_r ){ last;} # do not make several htmls for the same page - omits sorting
    }
  }
}
#----------------------------------------------------------
# get file finformation - size, date, and both raw
#----------------------------------------------------------
sub get_status {
  my $item=shift;
  my $plane_item=shift;
  my @status=((stat("$item/$plane_item"))[7,9]);
  my $size=sprintf "%.1f&nbsp;K",($status[0]/1024);
  my @time=(localtime($status[1]));
  my $month=('Jan','Feb','Mar','Apr','Mai','Jun','Jul','Aug','Sep','Okt','Nov','Dec')[$time[4]];
  return ($size,$time[3].' '.$month.' '.(1900+$time[5]),$status[1],$status[0]);
}

#----------------------------------------------------------
# creates the index for the java script offline search engine
#----------------------------------------------------------
sub add_to_js_index {
  my $linkname=shift;
  my $path=shift;
  my $filename=shift;
  my $age=shift;
  my $size=shift;
  my $im_marker=shift;
  my $tag=shift;
  my $inf=shift;
  my ($text,$words);
  $linkname=~tr/\"\\\r\n//d;
  $filename=~tr/\"//;
  if ( $has_pdftotext && $filename =~ /\.pdf$/mi) {
    process_format($filename,$linkname,\$words,\$text,"pdftotext \"$path/$filename\" -");
  }
  elsif ($filename =~ /\.x?html?$|\.xml$/mi) {
    my $raw_html;
    read_document($path,$filename,\$raw_html);
    my $hs;
    if ( $use_extract ) {
      my $hs = HTML::ExtractContent->new;
      $words=$hs->extract($raw_html)->as_text;
    } else {
      $hs = HTML::Strip->new();
      $words=$hs->parse( $raw_html );
    }
    undef $hs;
    process_indextext($linkname,$filename,\$words,\$text);
  } elsif ($filename =~ /\.txt$|\.ini$|\.cfg$|\.pl$|\.pm$/i ) { # text files
    read_document($path,$filename,\$words);
    process_indextext($linkname,$filename,\$words,\$text);
  } elsif ( $has_catdoc && $filename =~ /\.doc$/mi) {
    process_format($filename,$linkname,\$words,\$text,"catdoc \"$path/$filename\"");
  } elsif ( $has_catdoc && $filename =~ /\.ppt$/mi) {
    process_format($filename,$linkname,\$words,\$text,"catppt \"$path/$filename\"");
  } elsif ( $has_catdoc && $filename =~ /\.xls$/mi) {
    process_format($filename,$linkname,\$words,\$text,"xls2csv \"$path/$filename\"");
  } elsif ( $has_mp3info && $filename =~ /\.mp3$/mi) {
    process_mp3($path,$filename,$linkname,\$words,\$text,$tag,$inf);
  }
  else { $text="$linkname $filename"; $words="$linkname $filename"; clean_words(\$words) }
  if ( $k ){ print INDEX "c++; "}
  if ( $im_marker ) {
     my $fn=get_zero_lead($i).".html";
     print INDEX "item[c]=new Array (\"$size\",\"thumb/$fn\",\"$words\",\"$text\",\"$linkname\",\"$age\");\n";
  } else {
     print INDEX "item[c]=new Array (\"$size\",\".$path/$filename\",\"$words\",\"$text\",\"$linkname\",\"$age\");\n";
  }
  $k++;
}

#----------------------------------------------------------
# read a document an place the stream into file reference
#----------------------------------------------------------
sub read_document {
  my $path=shift;
  my $filename=shift;
  my $fileref=shift;
  open (IN,"<$path/$filename")||die "Could'nt open $path/$filename - $!";
  read  IN,$$fileref,262144;
  close (IN);
}
#----------------------------------------------------------
# get info from MP3 tags and place into search info
#----------------------------------------------------------
sub process_mp3 {
  my $path=shift;
  my $filename=shift;
  my $linkname=shift;
  my $wordsref=shift;
  my $textref=shift;
  my $tag=shift;
  my $inf=shift;
  my ($stereo,$vbr);
  if ( ! $tag->{TITLE} ) {$tag->{TITLE}="?";}
  if ( ! $tag->{ARTIST} ) {$tag->{ARTIST}="?";}
  if ( ! $tag->{ALBUM} ) {$tag->{ALBUM}="?";}
  if ( ! $tag->{YEAR} ) {$tag->{YEAR}="?";}
  if ( ! $tag->{COMMENT} ) {$tag->{COMMENT}="?";}
  if ( ! $tag->{GENRE} ) {$tag->{GENRE}="?";}
  $$wordsref.="$tag->{TITLE} $tag->{ARTIST} $tag->{ALBUM} $tag->{YEAR} $tag->{COMMENT} $tag->{GENRE}";
  $$textref.="Title: $tag->{TITLE}, Artist: $tag->{ARTIST}, Album: $tag->{ALBUM}, ";
  $$textref.="Year: $tag->{YEAR}, Genre: $tag->{GENRE}, Comment: $tag->{COMMENT} ";
  if ( $inf->{STEREO} ) { $stereo="Stereo" } else { $stereo="Mono" }
  if ( $inf->{VBR} ) { $vbr="VBR" } else { $vbr="CBR" }
  if ( ! $inf->{MM} ) {$inf->{MM}="?";}
  if ( ! $inf->{SS} ) {$inf->{SS}="?";}
  if ( ! $inf->{BITRATE} ) {$inf->{BITRATE}="?";}
  if ( ! $inf->{FREQUENCY} ) {$inf->{FREQUENCY}="?";}
  $$textref.="Play time: $inf->{MM}:$inf->{SS}, Bitrate: $inf->{BITRATE} kbps $vbr $stereo, Freqency: $inf->{FREQUENCY} kHz";
  $$wordsref.=" $linkname $filename";
  clean_words($wordsref);
  clean_text($textref);

}
#----------------------------------------------------------
# run external program and get the text stream out of a document
#----------------------------------------------------------
sub process_format {
  my $filename=shift;
  my $linkname=shift;
  my $wordsref=shift;
  my $textref=shift;
  my $external_prg_call=shift;
  $$wordsref=substr(`$external_prg_call`,0,262144);
  process_indextext($linkname,$filename,$wordsref,$textref);
}

#----------------------------------------------------------
# prepare the provided text for output as a search result
#----------------------------------------------------------
sub process_indextext {
  my $linkname=shift;
  my $filename=shift;
  my $wordsref=shift;
  my $textref=shift;
  $$textref=substr($$wordsref,0,450).'...';
  $$wordsref.=" $linkname $filename";
  clean_words($wordsref);
  clean_text($textref);
}
#----------------------------------------------------------
# process the document text and deliver back a comma separated list of unique search words
#----------------------------------------------------------
sub clean_words {
  my $words=shift;
  $$words=~tr/0-9A-Za-z_\-/ /c;
  $$words=~tr/A-Z/a-z/;
  $$words=~s/( and | has | get | but | the | for | was | got )/ /go;
  my @words=split /\s/,$$words;
  @words=sort(@words);
  my ($oldword,@squeezed)=('');
  foreach my $word (@words) {
    if ( length($word) <= 2 ) {next}
    if ( $word ne $oldword ) { push @squeezed,($word) }
    $oldword=$word;
  }
  $$words=join ',',@squeezed;
}
#----------------------------------------------------------
# clean the introduction text from cr's and backslashes (which may escape other characters)
#----------------------------------------------------------
sub clean_text {
  my $text=shift;
  $$text=~tr/\\\"//d;
  $$text=~tr/\r\n/  /;
  $$text=~tr/ / /s;
}
#----------------------------------------------------------
# calculates the image thumbs and previews
#----------------------------------------------------------
sub process_image {
my $mgk=shift;
my $i=shift;
my $item=shift;
my $plane_item=shift;
my $aref_planecontent=shift;
my $ht_entries=shift;
$thumbfile=get_zero_lead($i);
print "processing image: $plane_item\n" if ! $opt_q;
my $nexthtm=get_zero_lead($i+1).'.html';
my $prevhtm=get_zero_lead($i-1).'.html';
my $thumbname=$thumbfile;
@$mgk=();
my ($width,$height,$newheight,$newwidth);
if ( $opt_n ){
  ($width,$height)=$mgk->Ping("$item/$plane_item");
  if ( ! $height||! $width ) { $height=$maxwidth; $width=$maxwidth }
  ($newheight,$newwidth)=calc_newsize($height,$width);
} else {
  $mgk->Set(colorspace=>'RGB');
  $mgk->Set(magick=>'jpg',quality=>40);
  $mgk->Read("$item/$plane_item");
  ($height,$width) = $mgk->Get('rows','columns');
  ($newheight,$newwidth)=calc_newsize($height,$width);
  $mgk->Thumbnail(geometry=>$newwidth.'x'.$newheight);
  $mgk->Write("$thumbdir/m_$thumbname.jpg");
  $mgk->Resize(geometry=>$thumbsize);
  $mgk->Write("$thumbdir/$thumbfile.jpg");
}
$thumbfile="../thumb/$thumbfile.jpg";
my ($linkname)=get_description($item,$plane_item,$ht_entries);
my $img=new CGI::FastTemplate($templatedir);
$img->define(main =>'image.tpl');
print "$item/$plane_item\n";
$img->assign(IMAGE   =>"../thumb/m_$thumbname.jpg",
             IMLINK  =>"../../$item/$plane_item",
             NAME    =>$linkname,
             PREV    =>$prevhtm,
             NEXT    =>$nexthtm,
             HEIGHT  =>$newheight,
             WIDTH   =>$newwidth);
$img->parse(MAIN =>"main");
my $imgfileref=$img->fetch("MAIN");
undef $img;
open (IMG,">$thumbdir/$thumbname.html")||die "Could'nt open $thumbdir/$thumbname.html - $!";
print IMG $$imgfileref;
close (IMG);
my ($size,$age,$rawsize,$rawage)=get_status($item,$plane_item);
push @$aref_planecontent, [$thumbfile,"$width&nbsp;X&nbsp;$height&nbsp;px",$age,$rawsize,$rawage,"../thumb/$thumbname.html",$linkname];
return ($linkname,$age,$size);
}

#----------------------------------------------------------
# get the file description
#----------------------------------------------------------
sub get_description {  # determine the file description
my $item=shift;
my $plane_item=shift;
my $ht_entries_ref=shift;
my $notitle=shift;
my ($tag,$inf);
my ($linkname)=check_htaccess($ht_entries_ref,$plane_item);
if ($has_pdftotext && $plane_item =~ /\.pdf$/i ) {
  my $info=`pdfinfo "$item/$plane_item"`;
  if ((! $linkname ) && (! $notitle ) && ( !$opt_x )) {
    $info=~/^Title:[  ]*([^\n]*)$/m;
    $linkname=$1;
  }
  $info=~/^Pages:[  ]*([^\n]*)$/m;
  $linkname.=' - '.$1.' Pages';
  if ( length($linkname) < 20 ){
    $linkname="$plane_item - ".$1.' Pages';
  }
} elsif ((! $linkname )&& $plane_item =~ /\.html*$|\.xml$/i ) {
  open (IN,"<$item/$plane_item")||die "Could'nt open $item/$plane_item - $!";
  read  IN,my $title,65536;
  close (IN);
  $title=~/<( |\n|\r\n)*title( |\n|\r\n)*>([^<]*)<( |\n|\r\n)*\/( |\n|\r\n)*title( |\n|\r\n)*>/i;
  $linkname=$3;
} elsif ( $has_mp3info && $plane_item =~ /\.mp3$/mi ) {
  $tag=MP3::Info::get_mp3tag("$item/$plane_item") or goto badmp3;
  $inf=MP3::Info::get_mp3info("$item/$plane_item");
  if (! $linkname ) {
     if ( $tag->{TITLE} ) {
       $linkname= $tag->{TITLE};
     } else { $linkname=$plane_item;
     }
  }
  $linkname.=" - $inf->{TIME} $inf->{BITRATE} kbps";
}
badmp3:
if ( ! $linkname ) {
  $linkname="$plane_item";
}
return ($linkname,$tag,$inf);
}

#----------------------------------------------------------
# calculate the size of the image
#----------------------------------------------------------
sub calc_newsize {
  my $height=shift;
  my $width=shift;
  my ($newheight,$newwidth);
    if ($width > $maxwidth) {
    $newheight=int($height/($width/$maxwidth));   # keep the aspect ratio
    $newwidth=$maxwidth;
  } else { $newwidth=$width; $newheight=$height }
  return($newheight,$newwidth);
}

#----------------------------------------------------------
# add leading zeroes to the image name
#----------------------------------------------------------
sub get_zero_lead {    # add leading zeros to the filename
  my $num=shift;
  $_='00000'.$num;
  /(.{6}$)/;
  return $1;
}

#----------------------------------------------------------
# tries to read the .htaccess file to get the description
#----------------------------------------------------------
sub check_htaccess {    # check if a description in .htaccess exists - this becomes the higherst priority
  my $htref=shift;
  my $file=shift;
  foreach my $line (@$htref) {
    if ( $line =~/^ *AddDescription *"([^"]*)" *$file/m ) {
      return ($1);
    }
  }
  return (0);
}

#----------------------------------------------------------
# select the belonging icon for the file type
#----------------------------------------------------------
sub get_icon {
  my $filename=shift;
  my $fake=shift;
     print "Processing file: $filename\n" if ! $opt_q && ! $fake;
  $filename=~/\.([^\.]*)$/;
  my $extension=$1;
  if ( ! $extension ) { $extension='dwg' }
  else {
    $extension=~tr/A-Z/a-z/;
    if ( ! $main::icons{$extension} ) {$extension='dwg'}
  }
  my $icon="../resources/$main::icons{$extension}";
  return $icon;
}
;1